#!@l_prefix@/bin/perl
##
##  registry-db.pl - OpenPKG registration database setup and maintenance
##  Copyright (c) 2000-2005 OpenPKG Foundation e.V. <http://openpkg.net/>
##  Copyright (c) 2000-2005 Ralf S. Engelschall <http://engelschall.com/>
##
##  Permission to use, copy, modify, and distribute this software for
##  any purpose with or without fee is hereby granted, provided that
##  the above copyright notice and this permission notice appear in all
##  copies.
##
##  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
##  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
##  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
##  IN NO EVENT SHALL THE AUTHORS AND COPYRIGHT HOLDERS AND THEIR
##  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
##  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
##  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
##  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
##  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
##  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
##  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
##  SUCH DAMAGE.
##

require 5.008;
use strict;

use Data::Dumper;
use DBI;
use DBD::Pg;

#   configure optional debugging
$Data::Dumper::Purity = 1;
$Data::Dumper::Indent = 1;
$Data::Dumper::Terse  = 1;

#   program name, version and date
my $progname="registry-db.pl";
my $progvers="0.0.15";
my $progdate="19-Nov-2005";

#   determine path to OpenPKG instance
my $PREFIX='@l_prefix@';
$PREFIX=$ENV{OPENPKG_PREFIX} if ($ENV{OPENPKG_PREFIX} ne "");

#   configuration
my $cfg = {};
$cfg->{db}->{host}="127.0.0.1";
$cfg->{db}->{template1}->{username}="postgresql";
$cfg->{db}->{template1}->{password}="postgresql";
$cfg->{db}->{registry}->{username}="registry";
$cfg->{db}->{registry}->{password}="registry";


#   catch and hold time
my $time = time();

#   database handle and scratch variables
#
my $sql; # scratch SQL string database operations
my $rv;  # scratch return value for database operations
my $rc;  # scratch return code for database operations
my $sth; # scratch statement handle for database operations
my $msg; # scratch variable for message fiddling
my $dbh; # database handle

sub dbinit($) {
    my ($dbname) = @_;

    return DBI->connect (
        "DBI:Pg:dbname=".$dbname.";host=".$cfg->{db}->{host},
        $cfg->{db}->{$dbname}->{username},
        $cfg->{db}->{$dbname}->{password},
        {
            PrintError => 0,
            RaiseError => 0,
            AutoCommit => 1
        }
    );
}

#   create user, tablespace and database
#
$dbh = &dbinit("template1");
die "ERROR:$0: DataBase error: ".$DBI::errstr."\n" if (not defined $dbh);

$sql="CREATE USER registry ENCRYPTED PASSWORD 'registry' NOCREATEDB NOCREATEUSER;";
$rv = $dbh->do($sql);
print "$sql\nrv=".$rv." message=".$dbh->errstr."\n\n";

$sql="CREATE TABLESPACE registry OWNER registry LOCATION '$PREFIX/var/openpkg-registry/db';";
$rv = $dbh->do($sql);
print "$sql\nrv=".$rv." message=".$dbh->errstr."\n\n";

$sql="CREATE DATABASE registry OWNER registry TABLESPACE registry;";
$rv = $dbh->do($sql);
print "$sql\nrv=".$rv." message=".$dbh->errstr."\n\n";

$dbh->disconnect(); undef $dbh;

#   registry
#
$dbh = &dbinit("registry");
die "ERROR:$0: DataBase error: ".$DBI::errstr."\n" if (not defined $dbh);

my $reg_config;
while(&upgrade()) {};

#   die gracefully
$dbh->disconnect(); undef $dbh;
exit(0);

sub upgrade()
{
    $sql="SELECT * FROM reg_config;";
    $rv = $dbh->selectall_hashref($sql, "key");
    if (not defined %{$rv}->{version}) {
        $reg_config->{ 'version' => { 'key' => 'version', 'value' => '0.0.0' } }
    }
    else {
        $reg_config=$rv;
    }

    if($reg_config->{version}->{value} =~ m/^([0-9]+)\.([0-9]+)\.([0-9]+)$/) {
        $reg_config->{version}->{major}=$1;
        $reg_config->{version}->{minor}=$2;
        $reg_config->{version}->{patch}=$3;
    }

    if($reg_config->{version}->{value} eq $progvers) {
        print "database=" . %{$rv}->{version}->{value} . " is at my version=" .  $progvers . "\n";
        return 0;
    }
    elsif ($reg_config->{version}->{major} == 0 and $reg_config->{version}->{minor} == 0 and $reg_config->{version}->{patch} == 0) {
        print "upgrading from version=" . %{$rv}->{version}->{value} . "\n";
        $dbh->{AutoCommit} = 0;
        $dbh->begin_work;
        $sql = &schema();
        $sql .= "INSERT INTO reg_config VALUES ( 'version', '".$progvers."' );";
        $sql .= &sample() if ($progvers =~ m/^0\.0\./);
        $rv = $dbh->do($sql);
        $dbh->commit;
        print "[schema]\nrv=".$rv." message=".$dbh->errstr."\n\n";
        $dbh->{AutoCommit} = 1;
        return 1;
    }
    elsif ($reg_config->{version}->{major} == 0 and $reg_config->{version}->{minor} == 0) {
        print "upgrading from version=" . %{$rv}->{version}->{value} . "\n";
        $sql .= "INSERT INTO reg_config VALUES ( 'version', '".$progvers."' );";
        $rv = $dbh->do($sql);
        print "$sql\nrv=".$rv." message=".$dbh->errstr."\n\n";
        return 0;
    }
    else {
        print "program version=" . $progvers . " cannot handle database version=" . %{$rv}->{version}->{value} . "\n";
        return 0;
    }
    die
}

sub schema()
{
    return <<'EOT'
    -- OpenPKG configuration
    CREATE TABLE reg_config (
        key           TEXT
                      UNIQUE NOT NULL,
                      -- generic key
                      -- [version]
        value         TEXT
                      NOT NULL,
                      -- generic value
                      -- [0.8.15]
        mtime         TIMESTAMP (0) WITHOUT TIME ZONE
                      NOT NULL
                      DEFAULT CURRENT_TIMESTAMP,
                      -- time of last modification
                      -- [2005-11-08 00:00:00]
        PRIMARY KEY (key)
    );

    -- OpenPKG user
    CREATE TABLE reg_user (
        username      TEXT
                      NOT NULL
                      CHECK(username ~ '^.+@.+$'),
                      -- OSSP ase user
                      -- [one@example.com]
        heartbeat     TIMESTAMP (0) WITHOUT TIME ZONE
                      NOT NULL
                      DEFAULT CURRENT_TIMESTAMP,
                      -- time of last registry.cgi usage
                      -- [2005-11-08 00:00:00]
        PRIMARY KEY (username)
    );

    -- OpenPKG instance
    CREATE TABLE reg_instance (
        uuid_registry TEXT
                      UNIQUE NOT NULL
                      CHECK (uuid_registry ~ '^[0-9a-f]{8}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{12}$'),
                      -- <prefix>/etc/openpkg/uuid:UUID_REGISTRY
                      -- [d98267da-0874-11d9-b8f6-0002a5489f14]
        uuid_instance TEXT
                      NOT NULL
                      CHECK (uuid_instance ~ '^[0-9a-f]{8}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{12}$'),
                      -- <prefix>/etc/openpkg/uuid:UUID_INSTANCE
                      -- [1169823e-22db-3547-9f72-20c2ef18619f]
        uuid_platform TEXT
                      NOT NULL
                      CHECK (uuid_platform ~ '^[0-9a-f]{8}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{12}$'),
                      -- <prefix>/etc/openpkg/uuid:UUID_PLATFORM
                      -- [4a27ce02-730e-3cbf-a3fd-c32042c5f619]
        registry_desc TEXT
                      NOT NULL,
                      -- human-friendly description
                      -- [openpkg://host.domain.invalid/openpkg]
        registry_user TEXT
                      NOT NULL
                      REFERENCES reg_user (username),
                      -- reference to associated user
                      -- [one@example.com]
        registry_plat TEXT
                      CHECK (registry_plat ~ '^[^-]+-[^-]+$'),
                      -- output from openpkg rpm --eval '%{l_platform -p}'
                      -- [ix86-freebsd5.4]
        registry_orel TEXT
                      CHECK (registry_orel ~ '^[^-]+-(CURRENT|[0-9]+\.[0-9]+)$'),
                      -- output from openpkg rpm --eval '%{l_openpkg_release}'
                      -- [OpenPKG-2.5]
        registry_date TIMESTAMP (0) WITHOUT TIME ZONE
                      NOT NULL
                      DEFAULT CURRENT_TIMESTAMP,
                      -- time of registration
                      -- [2005-11-08 00:00:00]
        associated    BOOLEAN
                      NOT NULL
                      DEFAULT 'false',
                      -- whether instance is associated to user
                      -- [true]
        PRIMARY KEY (uuid_registry)
    );

    -- Access Statistic
    CREATE TABLE reg_statistic (
        login         TEXT
                      NOT NULL
                      CHECK (   login ~ '^.+@.+'
                             OR login ~ '^[0-9a-f]{8}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{12}$'),
                      -- reference to either reg_user.username or instance.uuid_registry
                      -- [one@example.com] [d98267da-0874-11d9-b8f6-0002a5489f14]
        addr          TEXT
                      NOT NULL,
                      -- remote peer IP address
                      -- [1.2.3.4]
        time_first    TIMESTAMP (0) WITHOUT TIME ZONE
                      NOT NULL,
                      -- time login+addr was seen first
                      -- [2005-11-08 00:00:00]
        time_last     TIMESTAMP (0) WITHOUT TIME ZONE
                      NOT NULL,
                      -- time login+addr was seen last
                      -- [2005-11-08 00:00:00]
        count         INTEGER
                      NOT NULL
                      DEFAULT '0',
                      -- number login+addr was seen at all
                      -- [0]
        PRIMARY KEY (login, addr)
    );

EOT
}

sub sample()
{
    return <<'EOT'
    INSERT INTO reg_user VALUES ('one\@example.com');
    INSERT INTO reg_user VALUES ('two\@example.com');
    INSERT INTO reg_instance VALUES (
        'd98267da-0874-11d9-b8f6-0002a5489f14',
        '1169823e-22db-3547-9f72-20c2ef18619f',
        '4a27ce02-730e-3cbf-a3fd-c32042c5f619',
        'openpkg://one.example.com/current/inst/one',
        'one@example.com',
        'ix86-freebsd5.4',
        'OpenPKG-CURRENT'
    );
    INSERT INTO reg_instance VALUES (
        'de9b7b7e-996c-11d9-acc4-000f20f84e91',
        'f9212ca7-2ab9-339e-8cf3-e32b9dcad48b',
        '86bec982-9ea3-3992-9c91-6775c77bb102',
        'openpkg://two.example.com/release/inst/two',
        'two@example.com',
        'ix86-freebsd5.4',
        'OpenPKG-2.5'
    );
EOT
}

